package ru.ifmo.vizi.dmkm.net;

import ru.ifmo.vizi.base.Configuration;
import ru.ifmo.vizi.base.widgets.ShapeStyle;
import ru.ifmo.vizi.dmkm.NetEditor;
import ru.ifmo.vizi.dmkm.widgets.Arc;
import ru.ifmo.vizi.dmkm.widgets.Rect;
import ru.ifmo.vizi.dmkm.widgets.Vertex;

import java.awt.*;

/**
 * Net is a container for all Shapes such as Vertices and Arcs.
 * @author Yuri Bedny
 * @version 0.0
 */
public class Net extends Container {

    /**
     * Styles for vertices.
     */
    private ShapeStyle vertexStyles[];

    /**
     * Styles for arcs.
     */
    private ShapeStyle arcStyles[];

    /**
     * Styles for rects.
     */
    private ShapeStyle rectStyles[];

    /**
     * Styles for circles.
     */
    private ShapeStyle cirStyles[];

    /**
     * Diameter of the vertex.
     */
    private int vertexDiam;

    /**
     * Diameter of the vertex in percent.
     */
    private int vertexDiamPersent;

    /**
     * Diameter of the circle.
     */
    private int circleDiam;

    /**
     * Diameter of the circle in percent.
     */
    private int circleDiamPersent;

    /**
     * Number of the vertices.
     */
    private int n;

    /**
     * Adjacency matrix.
     */
    private int g[][];

    /**
     * Flow.
     */
    private int f[][];

    /**
     * Whether to show the flow.
     */
    private boolean isFlow;

    /**
     * Potentials of the vertices.
     */
    private int p[];

    /**
     * Whether to show potentials.
     */
    private boolean isPotentials;

    /**
     * Whether to show selected arcs.
     */
    private boolean isNewArc;

    /**
     * Whether vertex has been deleted.
     */
    private boolean del[];

    /**
     * Vertex with minimum potential.
     */
    private int minv;

    /**
     * Whether to show vertex with min potential.
     */
    private boolean isMinv;

    /**
     * Vertices.
     */
    private Vertex vertices[];

    /**
     * Selected arcs.
     */
    private boolean newArc[][];

    /**
     * Layout manager for vertices, arcs, circles and rects.
     */
    private VertexLayouter layouter;

    /**
     * Editor for net.
     */
    private NetEditor editor;

    /**
     * Capacity of the main arcs in net.
     */
    private int mainNetCapacity[][];

    /**
     * Flow in main net.
     */
    private int mainNetFlow[][];

    /**
     * Vizi's configuration.
     */
    private Configuration config;

    /**
     * Container for arcs.
     */
    private Arc arcs[];


    /**
     * Returns vertex layouter.
     * @return vertex layouter for current net.
     */
    public VertexLayouter getVertexLayouter() {
        return layouter;
    }

    /**
     * Adds vertices to the net.
     */
    private void paintVertices() {
        vertices = new Vertex[n];
        for (int i = 0; i < n; i++) {

            Vertex v = new Vertex(vertexStyles, vertexDiam);
            v.setLocation(layouter.getLocation(i));
            if (i == 0) {
                v.setMessage(config.getParameter("source-caption"));
            } else if (i == n - 1) {
                v.setMessage(config.getParameter("sink-caption"));
            } else
                v.setMessage(Integer.toString(i));
            v.adjustFontSize();
            v.setStyle(0);
            if (isMinv) if (minv == i) v.setStyle(1);

            //   
            if (del[i]) v.setStyle(2);

            vertices[i] = v;
        }

        for (int i = 0; i < n; i++)
            add(vertices[i]);

    }

    /**
     * Adds rects to the net.
     */
    private void paintPotentials() {
        for (int i = 0; i < n; i++)
            if (!del[i]) {
                Rect rect = new Rect(rectStyles, vertexDiam / 2, vertexDiam / 2, Integer.toString(p[i]));
                rect.adjustFontSize();
                rect.setLocation(vertices[i].getLocation().x + vertexDiam / 4, vertices[i].getLocation().y - vertexDiam / 2 - 1);
                if (isMinv && minv == i) rect.setStyle(1);
                add(rect);
            }
    }

    /**
     * Sets number of vertices in net.
     * @param n number of vertices to set.
     */
    public void setNumOfVertices(int n) {
        this.n = n;
        del = new boolean[n];
        layouter.setNumOfVertices(n);
    }

    /**
     * Gets styles from config.
     */
    private void getStyles() {
        vertexStyles = ShapeStyle.loadStyleSet(config, "vertex");
        arcStyles = ShapeStyle.loadStyleSet(config, "arc");
        cirStyles = ShapeStyle.loadStyleSet(config, "circle");
        rectStyles = ShapeStyle.loadStyleSet(config, "rect");
    }

    /**
     * Creates net - container for Shapes such as vertex and arc.
     * @param config vis confifuration.
     * @param editor net editor
     */
    public Net(Configuration config, NetEditor editor) {
        vertexDiamPersent = config.getInteger("vertex-diameter-percent");
        circleDiamPersent = config.getInteger("circle-diameter-percent");
        this.config = config;
        this.editor = editor;
        getStyles();
        layouter = new VertexLayouter(n, vertexDiam);
    }

    /**
     * Sets capacity and delete vertices.
     * @param g capacity.
     * @param del delete vertices.
     */
    public void setNet(int g[][], boolean del[]) {
        this.g = g;
        this.del = del;
    }

    /**
     * Sets capacity.
     * @param g capacity.
     */
    public void setNet(int g[][]) {
        this.g = g;
        del = new boolean[n];
    }

    /**
     * Sets flow.
     * @param f flow.
     * @param isFlow if this flag equals true flow will be displayed.
     */
    public void setFlow(int f[][], boolean isFlow) {
        this.f = f;
        this.isFlow = isFlow;
        this.isNewArc = false;
    }

    /**
     * Sets flow.
     * @param f flow
     * @param isFlow if this flag equals true flow will be displayed.
     * @param newArc selected arcs.
     */
    public void setFlow(int f[][], boolean isFlow, boolean newArc[][]) {
        this.f = f;
        this.isFlow = isFlow;
        this.newArc = newArc;
        this.isNewArc = true;
    }


    /**
     * Invoked when vis data changed.
     * @param g current capacity
     * @param f current flow
     */
    public void updateAppletData(int g[][], int f[][]) {
        mainNetCapacity = g;
        mainNetFlow = f;
    }

    /**
     * Add arc from vertices number i and j.
     * @param i the first vertex.
     * @param j the second vertex.
     * @param enabled whether the arc belongs to the main net.
     */
    private void paintArc(int i, int j, boolean enabled) {
        Point p1 = new Point(layouter.getVertexCenter(i));
        Point p2 = new Point(layouter.getVertexCenter(j));

        Arc arc;
        if (enabled) {
            arc = new Arc(arcStyles, vertexDiam, p1, p2, mainNetCapacity[i][j], mainNetFlow[i][j]);
            arc.setStyle(2);
        } else {
            if (isFlow)
                arc = new Arc(arcStyles, vertexDiam, p1, p2, g[i][j], f[i][j]);
            else
                arc = new Arc(arcStyles, vertexDiam, p1, p2, g[i][j]);
            if (isNewArc) if (newArc[i][j]) arc.setStyle(1);
        }
        arc.setSize(getSize());
        int minSize = Math.min(getSize().width, getSize().height);
        arc.setHeadSize(
                (int) (minSize * config.getDouble("arc-length-percent") / 100),
                (int) (minSize * config.getDouble("arc-width-percent") / 100)
        );

        Vertex cir = new Vertex(cirStyles, circleDiam, arc.getText(), i, j);
        cir.adjustFontSize();
        cir.setLocation(arc.getTextLocation(circleDiam));
        cir.setStyle(arc.getStyle());
        if (editor.isActive()) {
            cir.addMouseMotionListener(editor);
            cir.addMouseListener(editor);
        }
        add(cir);

        Arc temp[] = new Arc[arcs.length];
        for (int k = 0; k < arcs.length; k++) {
            temp[k] = arcs[k];
        }
        arcs = new Arc[arcs.length + 1];
        for (int k = 0; k < arcs.length - 1; k++) {
            arcs[k] = temp[k];
        }
        arcs[arcs.length - 1] = arc;
    }

    /**
     * Adds arcs to the net.
     * @param enabled whether the arc belongs to the main net.
     */
    private void paintArcs(boolean enabled) {
        int i, j;
        if (enabled) {
            for (i = 0; i < n; i++)
                for (j = 0; j < n; j++)
                    if (mainNetCapacity[i][j] > 0 && (del[i] || del[j] || g[i][j] == 0 && g[j][i] == 0))
                        paintArc(i, j, true);
        } else {
            for (i = 0; i < n; i++)
                if (!del[i])
                    for (j = 0; j < n; j++)
                        if (!del[j])
                            if (g[i][j] > 0) paintArc(i, j, false);

        }
    }


    /**
     * Adds vertices and arcs to network.
     */
    public void paint() {

        layouter.setSize(getSize());

        /**
         * Sets diameter of circles and vertices.
         */
        vertexDiam = Math.min(getSize().width, getSize().height) * vertexDiamPersent / 100;
        layouter.setVertexDiameter(vertexDiam);
        circleDiam = Math.min(getSize().width, getSize().height) * circleDiamPersent / 100;

        /**
         * Deletes components from the net.
         */
        Component cmp[] = getComponents();
        for (int i = 0; i < cmp.length; i++)
            if (cmp[i] instanceof Vertex || cmp[i] instanceof Arc || cmp[i] instanceof Rect)
                remove(cmp[i]);

        /**
         * Adds vertices.
         */
        paintVertices();

        /**
         * Adds potentials.
         */
        if (isPotentials) paintPotentials();

        /**
         * Adds selected circles.
         */
        arcs = new Arc[0];
        paintArcs(false);

        /**
         * Adds selected arcs.
         */
        for (int i = 0; i < arcs.length; i++) {
            add(arcs[i]);
        }

        /**
         * Adds enabled circles.
         */
        arcs = new Arc[0];
        paintArcs(true);

        /**
         * Adds enabled arcs.
         */
        for (int i = 0; i < arcs.length; i++) {
            add(arcs[i]);
        }

    }

    /**
     * Sets the potential of the vertices.
     * @param p potentials to set.
     * @param minv vertex with minimum potential.
     */
    public void setPotentials(int p[], int minv) {
        this.p = p;
        isPotentials = true;
        this.minv = minv;
        isMinv = true;
    }

    /**
     * Sets the potential of the vertices.
     * @param p potentials to set.
     */
    public void setPotentials(int p[]) {
        this.p = p;
        isPotentials = true;
        isMinv = false;
    }

    /**
     * Define if potentials need to be displayed.
     * @param isPotentials if this flag equals true potentials of the vertices will be displayed.
     */
    public void setPotentials(boolean isPotentials) {
        this.isPotentials = isPotentials;
        this.isMinv = false;
    }

}


