package ru.ifmo.vizi.dmkm.widgets;
import ru.ifmo.vizi.base.widgets.Shape;
import ru.ifmo.vizi.base.widgets.ShapeStyle;

import java.awt.*;

/**
 * Base class for arcs and edges.
 */
public abstract class Arrow extends Shape {

    /**
     * Default length.
     */
    private static final int DEFAULT_HEAD_LENGTH = 15;

    /**
     * Default width.
     */
    private static final int DEFAULT_HEAD_WIDTH = 4;

    /**
     * First arrow's point.
     */
    protected Point startPoint;

    /**
     * Second arrow's point.
     */
    protected Point secondPoint;

    /**
     * Arrow's head length.
     */
    private int headLength;

    /**
     * Arrow's head width.
     */
    private int headWidth;

    private boolean drawArrowAtStart;
    private boolean drawArrowAtEnd;

    public Arrow(ShapeStyle[] styleSet, Point startPoint, Point secondPoint,
                 int headLength, int headWidth,
                 boolean drawArrowAtStart, boolean drawArrowAtEnd)
    {
        super(styleSet);
        this.startPoint = startPoint;
        this.secondPoint = secondPoint;
        this.headLength = headLength;
        this.headWidth = headWidth;
        this.drawArrowAtStart = drawArrowAtStart;
        this.drawArrowAtEnd = drawArrowAtEnd;
    }

    /**
     * Creates an arrow with specified style set, startPoint and secondPoint points to draw, and
     * specified drawArrow parameters.
     *
     * @param styleSet arrow's style set
     * @param first startPoint point
     * @param second secondPoint point
     * @param drawArrowAtStart whether to draw arrow head at start point
     * @param drawArrowAtEnd whether to draw arrow head to end point
     */
    public Arrow(ShapeStyle[] styleSet, Point first, Point second, boolean drawArrowAtStart, boolean drawArrowAtEnd) {
        this(styleSet, first, second, DEFAULT_HEAD_LENGTH, DEFAULT_HEAD_WIDTH, drawArrowAtStart, drawArrowAtEnd);
    }

    /**
     * Creates an arrow with specified style set, startPoint and secondPoint points to draw.
     *
     * @param styleSet arrow's style set.
     * @param first startPoint point.
     * @param second secondPoint point.
     */
    public Arrow(ShapeStyle[] styleSet, Point first, Point second) {
        this(styleSet, first, second, false, true);
    }

    /**
     * Resizes the arrow's head so that it has length headLength and width headWidth.
     * @param headLength head length.
     * @param headWidth head width.
     */
    public void setHeadSize(int headLength, int headWidth) {
        this.headLength = headLength;
        this.headWidth = headWidth;
    }

    /**
     * Paints the arrow.
     * @param g current Graphics.
     */
    public void paint(Graphics g) {
        int x1 = startPoint.x;
        int y1 = startPoint.y;
        int x2 = secondPoint.x;
        int y2 = secondPoint.y;
        double arrowLength = dist(startPoint, secondPoint);
        double part = headLength / arrowLength;
        int hx1 = x1 + (int) ((x2 - x1) * part);
        int hy1 = y1 + (int) ((y2 - y1) * part);
        int hx2 = x2 + (int) ((x1 - x2) * part);
        int hy2 = y2 + (int) ((y1 - y2) * part);

        boolean closed = look.getBorderStatus(style) || look.getFillStatus(style);
        g.setColor(look.getBorderColor(style));
        g.drawLine(hx1, hy1, hx2, hy2);
        if (drawArrowAtStart) {
            paintHead(g, secondPoint, startPoint);
        }
        if (!closed || !drawArrowAtStart) {
            g.setColor(look.getBorderColor(style));
            g.drawLine(x1, y1, hx1, hy1);
        }

        if (drawArrowAtEnd) {
            paintHead(g, startPoint, secondPoint);
        }
        if (!closed || !drawArrowAtEnd) {
            g.setColor(look.getBorderColor(style));
            g.drawLine(hx2, hy2, x2, y2);
        }

        super.paint(g);
    }

    /**
     * Paints arrow's head.
     * @param g current Graphics.
     * @param from source point.
     * @param to destination point.
     */
    private void paintHead(Graphics g, Point from, Point to) {
        int x1 = from.x;
        int y1 = from.y;
        int x2 = to.x;
        int y2 = to.y;
        double arrowLength = dist(startPoint, secondPoint);
        double part = headLength / arrowLength;
        int hx = x2 + (int)((x1 - x2) * part);
        int hy = y2 + (int)((y1 - y2) * part);
        int hx1, hy1, hx2, hy2;
        int dy = y2 - y1;
        int dx = x2 - x1;
        if (dy == 0) {
            hy1 = hy - headWidth;
            hy2 = hy + headWidth;
            hx1 = hx2 = hx;
        } else {
            double alpha = Math.atan((double)dx / dy);
            hx1 = hx + (int)(headWidth * Math.cos(alpha));
            hy1 = hy - (int)(headWidth * Math.sin(alpha));
            hx2 = hx - (int)(headWidth * Math.cos(alpha));
            hy2 = hy + (int)(headWidth * Math.sin(alpha));
        }
        if (look.getFillStatus(style)) {
            int x[] = {x2, hx1, hx2};
            int y[] = {y2, hy1, hy2};
            g.setColor(look.getFillColor(style));
            g.fillPolygon(x, y, 3);
        }
        g.setColor(look.getBorderColor(style));
        g.drawLine(hx1, hy1, x2, y2);
        g.drawLine(hx2, hy2, x2, y2);
        if (look.getBorderStatus(style)) {
            g.drawLine(hx1, hy1, hx2, hy2);
        }
    }

    /**
     * Calculates the distance between two points.
     * @param first startPoint point.
     * @param second secondPoint point.
     * @return distance between two points.
     */
    private double dist(Point first, Point second) {
        double dx = second.x - first.x;
        double dy = second.y - first.y;
        return Math.sqrt(dx * dx + dy * dy);
    }

}
